package com.perforce.hws.testing;

import com.perforce.hws.automation.AutomationConfig;
import com.perforce.hwsclient.AlphaApi;
import com.perforce.hwsclient.ApiClient;
import com.perforce.hwsclient.DefaultApi;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.annotations.DataProvider;

import java.io.IOException;

/**
 * Created by tjuricek on 12/9/15.
 */
public abstract class ClientProvider {
	/** Default P4 user. */
	public static final String DEFAULT_USER = "jdoe";
	/** P4 password for DEFAULT_USER. */
	public static final String DEFAULT_PASSWORD = "johndoe1A!";
	/** Super P4 user. */
	private static final String SUPER_USER = "super";
	/** P4 password for SUPER_USER. */
	private static final String SUPER_PASSWORD = "superuser1A!";
    /** The Constant logger. */
    private static final Logger LOGGER =
    		LoggerFactory.getLogger(ClientProvider.class);

    /** The config. */
    private static AutomationConfig config;

    /**
     * Sets the config.
     *
     * @param config the new config
     */
    public static void setConfig(final AutomationConfig config) {
        LOGGER.info("Using config: " + config);
        LOGGER.info("HWS Path: " + config.getLocalHws());
        ClientProvider.config = config;
    }

    /**
     * Gets the config.
     *
     * @return the config
     */
    public static AutomationConfig getConfig() {
    	if (config == null) {
    		String message = "Configuration is not loaded";
            LOGGER.error(message);
            throw new RuntimeException(message);
        }
        return config;
    }

    /**
     * Creates the p4 client.
     *
     * @param user the user
     * @param password the password
     * @return the default api
     * @throws IOException Signals that an I/O exception has occurred.
     */
    public static DefaultApi createP4Client(final String user,
    										final String password)
    												throws IOException {
    	getConfig();
        ApiClient apiClient = ApiClient.createWithTicket(
        		true, config.getLocalHws(), user, password);
        return apiClient.createDefaultService();
    }

    /**
     * Check for version.
     *
     * @return the string
     */
    public static String checkForVersion() {
    	getConfig();
        ApiClient apiClient = new ApiClient(true, config.getLocalHws());
        return apiClient.getStatus().getVersion();
    }

    /**
     * Checks if is supported.
     *
     * @return true, if is supported
     */
    public static boolean isSupported() {
    	getConfig();
        ApiClient apiClient = new ApiClient(true, config.getLocalHws());
        return apiClient.isSupported();
    }

    /**
     * Creates the.
     *
     * @param user the user
     * @param password the password
     * @param apiClasses the api classes
     * @return the object[][]
     */
    private static Object[][] create(final String user,
    								 final String password,
    								 final Class<?> ...apiClasses) {
    	getConfig();
    	try {
	    	ApiClient apiClient = ApiClient.createWithTicket(
	    			true, config.getLocalHws(), user, password);
	    	Object[][] retVal = new Object[1][apiClasses.length];
	    	int index = 0;
	    	for (Class<?> apiClass : apiClasses) {
	    		retVal[0][index++] = apiClient.createService(apiClass);
	    		
	    	}
	    	return retVal;
    	} catch (Exception e) {
    		LOGGER.error("Could not create API as " + user, e);
    		throw new RuntimeException(e);
    	}
    }
    
    /**
     * Creates the project client as jdoe.
     *
     * @return the object[][]
     * @throws IOException Signals that an I/O exception has occurred.
     */
    @DataProvider(name = "ProjectClientAsJDoe")
    public static Object[][] createProjectClientAsJdoe() throws IOException {
    	return create(DEFAULT_USER, DEFAULT_PASSWORD, DefaultApi.class);
    }
    
    /**
     * Creates the project client as jdoe with alpha.
     *
     * @return the object[][]
     * @throws IOException Signals that an I/O exception has occurred.
     */
    @DataProvider(name = "ProjectClientAsJDoeWithAlpha")
    public static Object[][] createProjectClientAsJdoeWithAlpha()
    		throws IOException {
    	return create(DEFAULT_USER, DEFAULT_PASSWORD, 
    				  DefaultApi.class, AlphaApi.class);
    }

    /**
     * Creates the project client as super.
     *
     * @return the object[][]
     * @throws IOException Signals that an I/O exception has occurred.
     */
    @DataProvider(name = "ProjectClientAsSuper")
    public static Object[][] createProjectClientAsSuper() throws IOException {
    	return create(SUPER_USER, SUPER_PASSWORD, DefaultApi.class);
    }

    /**
     * Creates the project client as super with alpha.
     *
     * @return the object[][]
     * @throws IOException Signals that an I/O exception has occurred.
     */
    @DataProvider(name = "ProjectClientAsSuperWithAlpha")
    public static Object[][] createProjectClientAsSuperWithAlpha()
    		throws IOException {
    	return create(SUPER_USER, SUPER_PASSWORD, DefaultApi.class, AlphaApi.class);
    }
}