package com.perforce.hws.testing;

import com.perforce.hws.automation.ProjectLocations;
import com.perforce.hws.util.ProcessHelpers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Created by tristan on 4/27/16.
 */
public class ExecRubyTest implements ProcessHelpers, ProjectLocations {

    static final Logger logger = LoggerFactory.getLogger(ExecRubyTest.class);

    private String clientDir;
    private String script;

    public ExecRubyTest(String clientDir, String script) {
        this.clientDir = clientDir;
        this.script = script;
    }

    @Test
    public void execRubyTest() throws IOException, InterruptedException {
        Path scriptPath = Paths.get(clientDir, script).toAbsolutePath();
        logger.info("Running Ruby test {}", script);
        try {
            String output = execOutOrFail(rubyCommand(), scriptPath.toString());
            logger.debug("output of test {}:\n{}", script, output);
        } catch (IllegalStateException e) {
            Assert.fail("Ruby test failed", e);
        }
    }

    /**
     * We use a bash wrapper to launch Ruby, since:
     *
     * 1. IDEs like IntelliJ running in OS X don't use the terminal environments
     * 2. We generally use a "system" RVM in EC, which also doesn't always load into the environment by default
     *
     * @return
     */
    public String rubyCommand() throws IOException {
        return Paths.get(projectRootOrFail(),
                "client-ruby", "rvm_ruby.sh")
                .toAbsolutePath().toString();
    }
}
