package com.perforce.hws.testing.deployment;

import org.testng.Assert;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.util.stream.Collectors;

/**
 * Created by tristan on 5/13/16.
 */
public interface ConfigHelpers {

    default void switchConfigToHttp(Path configPath) throws IOException {
        String converted = Files.lines(configPath)
                .map(l -> {
//                    logger.info("line {}", l);
                    if (l.contains("HWS_PORT: 443")) {
                        return "HWS_PORT: 80";
                    } else if (l.contains("ENABLE_HTTPS: true")) {
                        return "ENABLE_HTTPS: false";
                    } else if (l.contains("KEYSTORE_FILE:")) {
                        return "# KEYSTORE_FILE: ";
                    } else if (l.contains("KEYSTORE_PASSWORD:")) {
                        return "# KEYSTORE_PASSWORD: ";
                    } else if (l.contains("JWT_SIGNING_KEY:")) {
                        return "# JWT_SIGNING_KEY: ";
                    } else if (l.contains("TRUSTSTORE_FILE:")) {
                        return "# TRUSTSTORE_FILE: ";
                    } else if (l.contains("TRUSTSTORE_PASSWORD:")) {
                        return "# TRUSTSTORE_PASSWORD: ";
                    } else {
                        return l;
                    }
                })
                .collect(Collectors.joining("\n"));
        Files.write(configPath, converted.getBytes(), StandardOpenOption.TRUNCATE_EXISTING);
    }

    default void assertConfigStillHttp(Path configPath) throws IOException {
        Files.lines(configPath)
                .forEach(line -> {
                    if (line.startsWith("HWS_PORT")) {
                        Assert.assertTrue(line.contains("80"), "port not set to 80: " + line);
                    } else if (line.startsWith("ENABLE_HTTPS")) {
                        Assert.assertTrue(line.contains("false"), "https is not false: " + line);
                    }
                });

    }
}
