package com.perforce.hws.testing.deployment.rpm;

import com.perforce.hws.automation.DeploymentCleanup;
import com.perforce.hws.automation.NetworkUtil;
import com.perforce.hws.testing.deployment.ConfigHelpers;
import com.perforce.hws.util.TimingHelpers;
import com.perforce.hwsclient.ApiClient;

import org.testng.Assert;
import org.testng.annotations.Test;

import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Created by tristan on 5/13/16.
 */
public class UpgradeConfigTests implements
        ConfigHelpers,
        NetworkUtil,
        RpmDeployment,
        TimingHelpers {

    /**
     * This ensures that edits to the configuration file after installation are not overridden
     * by re-running the installation + configuration procedures again.
     */
    // Disabling for now, there were odd problems on the BNR Centos 6.2 machine -
    // it kept running out of disk space and I had no access to adjusting this.
//    @Test
    public void upgradeDoesNotOverwriteConfig() throws Exception {
        DeploymentCleanup deploymentCleanup = new DeploymentCleanup();
        deploymentCleanup.run();

        installOldRpmFile(baseRpmUpgradePath());

        // Let's reset the server to be non-https.
        Path configPath = Paths.get("/", "opt", "perforce", "helix-web-services", "etc", "helix-web-services.conf").toAbsolutePath();

        switchConfigToHttp(configPath);

        // Now that we've reconfigured, we need to restart
        // NOTE: This is an older version, so this service name is actually obsolete
        execOrFail("service", "helix_web_services", "restart");

        boolean isRunning = waitFor(10, 1000, () -> !isPortAvailable(80));
        Assert.assertTrue(isRunning, "HWS isn't started at port 80");

        upgradeRpmBuild();

        isRunning = waitFor(10, 1000, () -> !isPortAvailable(80));
        Assert.assertTrue(isRunning, "HWS isn't resstarted at port 80");

        // Read the configuration file.
        assertConfigStillHttp(configPath);

        deploymentCleanup.run();
    }
}
