package com.perforce.hws.testing.server;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.BranchCommand;
import com.perforce.hwsclient.models.BranchesCommand;
import com.perforce.hwsclient.models.CommandResponse;

import org.testng.annotations.Test;

import java.io.IOException;
import java.util.List;

import static java.lang.String.format;
import static java.util.Arrays.asList;
import static org.testng.Assert.*;

/**
 * Test methods under /branches
 */
public class PerforceBranchesTests implements StringUtils {

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class,
            groups = "no-cloud")
    public void branchCRUDLifecycle(DefaultApi api) throws IOException {
        String rand = randomString();

        String branchName = format("test-branch-%s", rand);

        BranchCommand branch = new BranchCommand();
        branch.setBranch(branchName);
        branch.setView(asList(format("//depot/main/%s/... //depot/dev/%s/...", branchName, branchName)));

        CommandResponse response = api.addBranch("localhost", branch);

        List<BranchesCommand> branches =
                api.getBranches("localhost");

        assertTrue(
                branches.stream().anyMatch(m -> m.getBranch().equals(branchName))
        );

        BranchCommand saved = api.getBranch("localhost", branchName);

        assertEquals(saved.getView(), branch.getView());

        BranchCommand toUpdate = new BranchCommand();
        toUpdate.setDescription(format("test %s", rand));

        api.updateBranch("localhost", branchName, toUpdate);

        BranchCommand updated = api.getBranch("localhost", branchName);

        assertEquals(updated.getDescription().toString().trim(),
                toUpdate.getDescription().toString().trim());

        api.deleteBranch("localhost", branchName);

        List<BranchesCommand> allBranches2 = api.getBranches("localhost");

        assertFalse(
                allBranches2.stream().anyMatch(m -> m.getBranch().equals(branchName))
        );
    }
}
