package com.perforce.hws.testing.server;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.DepotCommand;
import com.perforce.hwsclient.models.DepotsCommand;

import org.testng.annotations.Test;

import java.io.IOException;
import java.util.List;

import static java.lang.String.format;
import static org.testng.Assert.*;

/**
 * Methods related to /p4/vAPI/depots.
 */
public class PerforceDepotTests implements StringUtils {

    @Test(dataProvider = "ProjectClientAsSuper",
            dataProviderClass = ClientProvider.class)
    public void depotCRUDLifecycle(DefaultApi api) throws IOException {
        String rand = randomString();

        String depot = format("test-depot-%s", rand);

        DepotCommand depotSpec = new DepotCommand();
        depotSpec.setDepot(depot);
        depotSpec.setType("local");
        depotSpec.setMap(format("%s/...", depot));
        depotSpec.setDescription(format("test %s", rand));

        api.addDepot("localhost", depotSpec);

        List<DepotsCommand> depots =
                api.getDepots("localhost");

        assertTrue(
                depots.stream().anyMatch(m -> m.getDepot().equals(depot))
        );

        DepotCommand saved =
                api.getDepot("localhost", depot);

        assertEquals(saved.getDescription().toString().trim(),
                depotSpec.getDescription().toString().trim());

        DepotCommand toUpdate = new DepotCommand();
        toUpdate.setDescription(format("update %s", rand));

        api.updateDepot("localhost", depot, toUpdate);

        DepotCommand updated =
                api.getDepot("localhost", depot);

        assertEquals(updated.getDescription().toString().trim(),
                toUpdate.getDescription().toString().trim());

        api.deleteDepot("localhost", depot);

        List<DepotsCommand> allDepots2 =
                api.getDepots("localhost");

        assertFalse(
                allDepots2.stream().anyMatch(m -> m.getDepot().equals(depot))
        );
    }
}
