package com.perforce.hws.testing.server;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.CommandResponse;
import com.perforce.hwsclient.models.GroupCommand;
import com.perforce.hwsclient.models.GroupsCommand;

import org.testng.annotations.Test;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static java.lang.String.format;
import static org.testng.Assert.*;

/**
 * Test methods under /p4/:api/groups
 */
public class PerforceGroupsTests implements StringUtils {

    @Test(dataProvider = "ProjectClientAsSuper",
            dataProviderClass = ClientProvider.class)
    public void groupCRUDLifecycle(DefaultApi api) throws IOException {
        String rand = randomString();

        String group = format("test-group-%s", rand);

        GroupCommand testGroup = new GroupCommand();
        testGroup.setGroup(group);
        testGroup.setUsers(Arrays.asList("jdoe"));
        testGroup.setOwners(Arrays.asList("jdoe"));

        CommandResponse createResponse = api.addGroup("localhost", testGroup);

        List<GroupsCommand> allGroups =
                api.getGroups("localhost");

        assertTrue(
                allGroups.stream().anyMatch(m -> m.getGroup().equals(group))
        );

        GroupCommand saved =
                api.getGroup("localhost", group);

        assertEquals(saved.getGroup().toString().trim(),
                testGroup.getGroup().toString().trim());
        assertEquals(saved.getOwners(), testGroup.getOwners());
        assertEquals(saved.getUsers(), testGroup.getUsers());

        GroupCommand toUpdate = new GroupCommand();
        toUpdate.setUsers(Arrays.asList("mmustermann"));

        api.updateGroup("localhost", group, toUpdate);

        GroupCommand updated =
                api.getGroup("localhost", group);

        assertTrue(updated.getUsers().contains("mmustermann"));

        api.deleteGroup("localhost", group);

        List<GroupsCommand> allGroups2 =
                api.getGroups("localhost");

        assertFalse(
                allGroups2.stream().anyMatch(m -> m.getGroup().equals(group))
        );
    }

    @Test(dataProvider = "ProjectClientAsSuper",
            dataProviderClass = ClientProvider.class,
            groups = "has-ldap")
    public void groupCRUDLifecycleWithLDAPSettings(DefaultApi api) {
        String rand = randomString();

        String group = format("test-group-%s", rand);

        GroupCommand testGroup = new GroupCommand();
        testGroup.setGroup(group);
        testGroup.setUsers(Arrays.asList("jdoe"));
        testGroup.setOwners(Arrays.asList("jdoe"));
        testGroup.setLdapConfig("ldap-config");
        testGroup.setLdapSearchQuery("ldap-search-query");
        testGroup.setLdapUserAttribute("ldap-user-attribute");

        CommandResponse createResponse = api.addGroup("localhost", testGroup);

        List<GroupsCommand> allGroups =
                api.getGroups("localhost");

        assertTrue(
                allGroups.stream().anyMatch(m -> m.getGroup().equals(group))
        );

        GroupCommand saved =
                api.getGroup("localhost", group);

        assertEquals(saved.getGroup().toString().trim(),
                testGroup.getGroup().toString().trim());
        assertEquals(saved.getOwners(), testGroup.getOwners());
        assertEquals(saved.getUsers(), testGroup.getUsers());
        assertEquals(saved.getLdapConfig(), testGroup.getLdapConfig());
        assertEquals(saved.getLdapSearchQuery(), testGroup.getLdapSearchQuery());
        assertEquals(saved.getLdapUserAttribute(), testGroup.getLdapUserAttribute());

        GroupCommand toUpdate = new GroupCommand();
        toUpdate.setUsers(Arrays.asList("mmustermann"));

        api.updateGroup("localhost", group, toUpdate);

        GroupCommand updated =
                api.getGroup("localhost", group);

        assertTrue(updated.getUsers().contains("mmustermann"));

        api.deleteGroup("localhost", group);

        List<GroupsCommand> allGroups2 =
                api.getGroups("localhost");

        assertFalse(
                allGroups2.stream().anyMatch(m -> m.getGroup().equals(group))
        );
    }
}
