package com.perforce.hws.testing.server;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.ChangeCommand;
import com.perforce.hwsclient.models.CommandResponse;
import com.perforce.hwsclient.models.JobCommand;
import com.perforce.hwsclient.models.JobsCommand;

import org.testng.annotations.Test;

import java.io.IOException;
import java.util.List;

import static java.lang.String.format;
import static org.testng.Assert.*;

/**
 * Test methods under /p4/:api/jobs
 */
public class PerforceJobsTests implements StringUtils {

    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void jobCRUDLifecycle(DefaultApi api) throws IOException {
        String rand = randomString();

        String jobDescription = format("test-job-%s", rand);

        JobCommand jobCommand = new JobCommand();
        jobCommand.setJob("new");
        jobCommand.put("User", "jdoe");
        jobCommand.put("Description", jobDescription);
        jobCommand.put("Status", "open");

        CommandResponse createResults =
                api.addJob("localhost", jobCommand);

        String job = (String) createResults.getResults().get(0).get("job");

        List<JobsCommand> allJobs =
                api.getJobs("localhost");

        assertTrue(
                allJobs.stream().anyMatch(m -> m.getJob().equals(job))
        );

        JobCommand saved =
                api.getJob("localhost", job);

        assertEquals(saved.get("Description").toString().trim(),
                jobCommand.get("Description").toString().trim());

        JobCommand toUpdate = new JobCommand();
        toUpdate.put("Status", "closed");

        api.updateJob("localhost", job, toUpdate);

        JobCommand updated =
                api.getJob("localhost", job);

        assertEquals("closed", updated.get("Status"));

        api.deleteJob("localhost", job);

        List<JobsCommand> allJobs2 =
                api.getJobs("localhost");

        assertFalse(
                allJobs2.stream().anyMatch(m -> m.getJob().equals(job))
        );
    }

    /**
     * Creates a new job which we fix with the first changelist.
     *
     * @param api used to create the job and read the changelist
     * @throws IOException
     */
    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class)
    public void fixJobWithFirstChange(DefaultApi api) throws IOException {

        String rand = randomString();

        String jobDescription = format("test-job-%s", rand);

        JobCommand jobCommand = new JobCommand();
        jobCommand.setJob("new");
        jobCommand.put("User", "jdoe");
        jobCommand.put("Description", jobDescription);
        jobCommand.put("Status", "open");

        CommandResponse createResults =
                api.addJob("localhost", jobCommand);

        String job = (String) createResults.getResults().get(0).get("job");

        api.addJobFix("localhost", job, "1", null);

        JobCommand updated = api.getJob("localhost", job);

        assertEquals(updated.get("Status"), "closed");

        ChangeCommand updatedChange = api.getChange("localhost", "1");

        assertTrue(updatedChange.getJobs().contains(job));

        api.deleteJobFix("localhost", job, "1");

        ChangeCommand updatedChange2 = api.getChange("localhost", "1");

        assertFalse(updatedChange2.getJobs().contains(job));
    }
}
