package com.perforce.hws.testing.server;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.LabelCommand;
import com.perforce.hwsclient.models.LabelsCommand;

import org.testng.annotations.Test;

import java.io.IOException;
import java.util.List;

import static java.lang.String.format;
import static java.util.Arrays.asList;
import static org.testng.Assert.*;

/**
 * Created by tjuricek on 1/21/16.
 */
public class PerforceLabelsTests implements StringUtils {

    // Currently disabled in cloud because I'm not sure how stream labels work
    @Test(dataProvider = "ProjectClientAsJDoe",
            dataProviderClass = ClientProvider.class,
            groups = "no-cloud")
    public void labelCRUDLifecycle(DefaultApi api) throws IOException {
        String rand = randomString();

        String label = format("test-label-%s", rand);

        LabelCommand newLabel = new LabelCommand();
        newLabel.setLabel(label);
        newLabel.setDescription(label);
        newLabel.setView(asList(format("//depot/main/%s/...", label)));

        api.addLabel("localhost", newLabel);

        List<LabelsCommand> allLabels =
                api.getLabels("localhost");

        assertTrue(
                allLabels.stream().anyMatch(m -> m.getLabel().equals(label))
        );

        LabelCommand saved =
                api.getLabel("localhost", label);

        assertEquals(saved.getView(), newLabel.getView());

        LabelCommand toUpdate = new LabelCommand();
        toUpdate.setDescription(format("update %s", rand));

        api.updateLabel("localhost", label, toUpdate);

        LabelCommand updated =
                api.getLabel("localhost", label);

        assertEquals(updated.getDescription().toString().trim(),
                toUpdate.getDescription().toString().trim());

        api.deleteLabel("localhost", label);

        List<LabelsCommand> allLabels2 =
                api.getLabels("localhost");

        assertFalse(
                allLabels2.stream().anyMatch(m -> m.getLabel().equals(label))
        );
    }
}

