package com.perforce.hws.testing.server;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.ServerCommand;
import com.perforce.hwsclient.models.ServersCommand;

import org.testng.annotations.Test;

import java.io.IOException;
import java.util.List;

import static java.lang.String.format;
import static org.testng.Assert.*;

/**
 * Tests methods under /p4/:api/servers
 */
public class PerforceServersTests implements StringUtils {

    @Test(dataProvider = "ProjectClientAsSuper",
            dataProviderClass = ClientProvider.class)
    public void serverCRUDLifecycle(DefaultApi api) throws IOException {
        String rand = randomString();

        String server = format("testserver", rand);

        ServerCommand newServer = new ServerCommand();
        newServer.setServerID(server);
        newServer.setDescription(format("test %s", rand));
        newServer.setType("server");
        newServer.setServices("standard");

        api.addServer("localhost", newServer);

        List<ServersCommand> allServers =
                api.getServers("localhost");

        assertTrue(
                allServers.stream().anyMatch(m -> m.getServerID().equals(server))
        );

        ServerCommand saved =
                api.getServer("localhost", server);

        assertEquals(saved.getDescription().toString().trim(),
                newServer.getDescription().toString().trim());

        ServerCommand toUpdate = new ServerCommand();
        toUpdate.setDescription(format("update %s", rand));

        api.updateServer("localhost", server, toUpdate);

        ServerCommand updated =
                api.getServer("localhost", server);

        assertEquals(updated.getDescription().toString().trim(),
                toUpdate.getDescription().toString().trim());

        api.deleteServer("localhost", server);

        List<ServersCommand> allServers2 =
                api.getServers("localhost");

        assertFalse(
                allServers2.stream().anyMatch(m -> m.getServerID().equals(server))
        );
    }
}
