package com.perforce.hws.testing.server;

import com.perforce.hws.testing.ClientProvider;
import com.perforce.hws.util.StringUtils;
import com.perforce.hwsclient.DefaultApi;
import com.perforce.hwsclient.models.UserCommand;
import com.perforce.hwsclient.models.UsersCommand;

import org.testng.annotations.Test;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static java.lang.String.format;
import static org.testng.Assert.*;

/**
 * /p4/:api/users tests
 */
public class PerforceUsersTests implements StringUtils {

    @Test(dataProvider = "ProjectClientAsSuper",
            dataProviderClass = ClientProvider.class)
    public void userCRUDLifecycle(DefaultApi api) {
        String rand = randomString();

        String userName = format("test-user-%s", rand);

        UserCommand newUser = new UserCommand();
        newUser.setUser(userName);
        newUser.setFullName("Test User");
        newUser.setEmail(format("%s@example.com", userName));
        newUser.setJobView("Owned-By=" + userName + "&status=open");
        newUser.setReviews(Arrays.asList("//depot/dev/...", "//depot/main/..."));

        api.addUser("localhost", newUser);

        List<UsersCommand> allUsers = api.getUsers("localhost", null, null);

        assertTrue(
                allUsers.stream().anyMatch(m -> m.getUser().equals(userName))
        );

        UserCommand saved =
                api.getUser("localhost", userName);

        assertEquals(saved.getFullName(), newUser.getFullName());
        assertEquals(saved.getJobView(), "Owned-By=" + userName + "&status=open");
        assertEquals(saved.getReviews(), Arrays.asList("//depot/dev/...", "//depot/main/..."));

        UserCommand toUpdate = new UserCommand();
        toUpdate.setFullName(format("New Name %s", rand));

        api.updateUser("localhost", userName, toUpdate);

        UserCommand updated =
                api.getUser("localhost", userName);

        assertEquals(updated.getFullName(), toUpdate.getFullName());

        api.deleteUser("localhost", userName);

        List<UsersCommand> allUsers2 =
                api.getUsers("localhost", null, null);

        assertFalse(
                allUsers2.stream().anyMatch(m -> m.getUser().equals(userName))
        );
    }

    @Test(dataProvider = "ProjectClientAsSuper",
            dataProviderClass = ClientProvider.class)
    public void canListSystemUsers(DefaultApi api) {
        String rand = randomString();

        String userName = format("system-test-user-%s", rand);

        UserCommand sysUser = new UserCommand();
        sysUser.setUser(userName);
        sysUser.setType("service");
        sysUser.setEmail(userName + "@example.com");
        sysUser.setFullName(userName);

        api.addUser("localhost", sysUser);

        List<UsersCommand> nonSysUsers = api.getUsers("localhost", null, null);
        assertFalse(nonSysUsers.stream().anyMatch(m -> m.getUser().equals(userName)));

        List<UsersCommand> sysUsers = api.getUsers("localhost", true, null);
        assertTrue(sysUsers.stream().anyMatch(m -> m.getUser().equals(userName)));
    }
}

